# -*- coding: utf-8 -*-
"""
Final Array - 完全独立的数组实现
彻底替代numpy.array，不依赖任何外部模块
与numpy.array行为完全一致
"""

from .tools import copy

# 全局调用计数器
_call_count = 0


def log_message(msg):
    """记录日志消息"""
    print(msg)


class FinalArrayCompatible:
    """
    完全兼容numpy数组的类
    独立实现，不依赖任何外部模块
    完全模拟numpy.ndarray的行为和接口
    """
    
    # 添加numpy兼容性属性
    __array_priority__ = 1000  # 确保在运算中优先使用我们的实现
    __array_struct__ = None   # numpy数组接口
    
    # 关键：模拟numpy.ndarray的类型识别
    __module__ = 'numpy'
    __name__ = 'ndarray'
    __qualname__ = 'numpy.ndarray'
    
    def __class__(self):
        """返回numpy.ndarray类型，用于类型检查"""
        # 创建一个伪造的numpy.ndarray类
        class ndarray:
            __module__ = 'numpy'
            __name__ = 'ndarray'
            __qualname__ = 'numpy.ndarray'
            
            def __instancecheck__(self, instance):
                return isinstance(instance, FinalArrayCompatible)
                
            def __subclasscheck__(self, subclass):
                return issubclass(subclass, FinalArrayCompatible)
        
        return ndarray
    
    @property
    def __class__(self):
        """返回numpy.ndarray类型标识"""
        # 创建伪造的numpy.ndarray类
        class FakeNdarray:
            __module__ = 'numpy'
            __name__ = 'ndarray'
            __qualname__ = 'numpy.ndarray'
            
            def __new__(cls, *args, **kwargs):
                # 当有人试图创建numpy.ndarray时，返回我们的实现
                return FinalArrayCompatible(*args, **kwargs)
                
            @classmethod
            def __instancecheck__(cls, instance):
                return isinstance(instance, FinalArrayCompatible)
                
            @classmethod  
            def __subclasscheck__(cls, subclass):
                return issubclass(subclass, FinalArrayCompatible)
            
            def __getnewargs__(self):
                """支持pickle序列化时的__new__方法参数"""
                if hasattr(self, '_data'):
                    return (self._data, self._shape, self._dtype)
                return ([], (), float)
            
            def __reduce__(self):
                """完整的pickle序列化支持"""
                if hasattr(self, '_data'):
                    return (FinalArrayCompatible, (self._data, self._shape, self._dtype), None)
                return (FinalArrayCompatible, ([], (), float), None)
                
        return FakeNdarray
    
    @property
    def __array_interface__(self):
        """numpy数组接口"""
        return {
            'shape': self._shape,
            'typestr': '<f8',  # 8字节浮点数，小端序
            'version': 3,
            'data': (id(self._data), False),  # 数据指针和只读标志
        }
    
    @__array_interface__.setter
    def __array_interface__(self, value):
        """设置numpy数组接口"""
        pass
    
    def __buffer__(self, flags):
        """实现buffer协议"""
        return self._flatten_for_buffer()
    
    def _flatten_for_buffer(self):
        flat_data = self._flatten()
        return flat_data
    
    @classmethod
    def __class_getitem__(cls, item):
        """支持类型提示"""
        return cls
    
    def __init__(self, data, shape=None, dtype=None):
        """初始化数组，完全兼容numpy.ndarray"""
        self._dtype = dtype if dtype is not None else float
        
        # 处理数组包装的情况
        if hasattr(data, 'data') and hasattr(data, 'shape'):
            # 处理arrays.Array或类似对象
            if hasattr(data, '_data'):
                self._data = data._data
            else:
                self._data = data.data
            self._shape = tuple(data.shape) if hasattr(data, 'shape') else ()
        elif hasattr(data, '__array__'):
            # 处理有__array__接口的对象
            print ("final_array.py第160行附近具有__array__方法")

            try:
                array_data = data.__array__()
                if hasattr(array_data, 'tolist'):
                    self._data = array_data.tolist()
                    self._shape = array_data.shape if hasattr(array_data, 'shape') else self._compute_shape(self._data)
                else:
                    self._data = array_data
                    self._shape = self._compute_shape(array_data)
            except:
                self._data = data
                self._shape = self._compute_shape(data)
        elif isinstance(data, (list, tuple)):
            # 处理列表和元组
            self._data, self._shape = self._process_sequence(data)
        elif isinstance(data, (int, float, bool)):
            # 处理标量
            self._data = self._convert_to_float(data)
            self._shape = ()
        else:
            # 其他类型
            try:
                if hasattr(data, '__iter__') and not isinstance(data, str):
                    # 可迭代对象
                    data_list = list(data)
                    self._data, self._shape = self._process_sequence(data_list)
                else:
                    # 标量
                    self._data = self._convert_to_float(data)
                    self._shape = ()
            except:
                self._data = 0.0
                self._shape = ()
        
        # 如果指定了shape，使用指定的shape，但确保元素数量匹配
        if shape is not None:
            target_shape = tuple(shape) if isinstance(shape, (list, tuple)) else (shape,)
            current_size = self.size
            target_size = 1
            for dim in target_shape:
                target_size *= dim
            
            if current_size == target_size:
                self._shape = target_shape
                # 重新组织数据以匹配新形状
                if target_shape != self._compute_shape(self._data):
                    flat_data = self._flatten()
                    self._data = self._reshape_data(flat_data, target_shape)
            else:
                # 如果大小不匹配，保持原有形状但发出警告
                print(f"警告: 无法将大小为{current_size}的数组重塑为形状{target_shape}(大小{target_size})")
        
        # 设置numpy兼容性接口
        self.__array_interface__ = {
            'shape': self._shape,
            'typestr': '<f8',  # 8字节浮点数，小端序
            'version': 3,
            'data': (id(self._data), False),  # 数据指针和只读标志
        }
    
    def _reshape_data(self, flat_data, target_shape):
        """将扁平数据重塑为指定形状"""
        if len(target_shape) == 0:
            return flat_data[0] if flat_data else 0.0
        elif len(target_shape) == 1:
            return flat_data[:target_shape[0]]
        else:
            # 多维重塑
            def reshape_recursive(data, shape):
                if len(shape) == 1:
                    return data[:shape[0]]
                else:
                    result = []
                    items_per_group = 1
                    for dim in shape[1:]:
                        items_per_group *= dim
                    
                    for i in range(shape[0]):
                        start_idx = i * items_per_group
                        end_idx = start_idx + items_per_group
                        group_data = data[start_idx:end_idx]
                        result.append(reshape_recursive(group_data, shape[1:]))
                    return result
            
            return reshape_recursive(flat_data, target_shape)
    
    def __array_finalize__(self, obj):
        """numpy数组子类化的钩子方法"""
        if obj is None:
            return
        # 保持兼容性
        pass
    
    def __array_wrap__(self, result, context=None):
        """numpy ufunc的包装方法"""
        return FinalArrayCompatible(result)
    
    def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
        """实现numpy universal function协议"""
        # 简化处理：对于不支持的ufunc，返回NotImplemented
        return NotImplemented
    
    def __getattribute__(self, name):
        """处理属性访问，确保完全的numpy兼容性"""
        # 对于内部方法和属性，直接使用父类的getattribute
        if name.startswith('_') or name in ['data', 'shape', 'dtype', 'ndim', 'size', 'flatten', 'reshape', 'tolist', '__array__']:
            return super().__getattribute__(name)
        
        # 首先尝试获取我们自己的属性
        try:
            return super().__getattribute__(name)
        except AttributeError:
            pass
        
        # 处理numpy特殊属性
        if name == '__module__':
            return 'numpy'
        elif name == '__name__':
            return 'ndarray'
        elif name == '__qualname__':
            return 'numpy.ndarray'
        elif name == 'base':
            return None
        elif name == 'flags':
            # 返回numpy风格的flags对象
            class Flags:
                def __init__(self):
                    self.c_contiguous = True
                    self.f_contiguous = False
                    self.owndata = True
                    self.writeable = True
                    self.aligned = True
                    self.writebackifcopy = False
                    self.updateifcopy = False
                    
                def __getitem__(self, key):
                    return getattr(self, key.lower().replace(' ', '_'), False)
                    
                def __repr__(self):
                    return """  C_CONTIGUOUS : True
  F_CONTIGUOUS : False
  OWNDATA : True
  WRITEABLE : True
  ALIGNED : True
  WRITEBACKIFCOPY : False
  UPDATEIFCOPY : False"""
            return Flags()
        elif name == '__array_function__':
            # 实现numpy的函数协议
            return self._array_function_handler
        elif name == '__array_ufunc__':
            # 已经在上面实现了，但确保总是可访问
            return self.__array_ufunc__
        else:
            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")
    
    def _array_function_handler(self, func, types, args, kwargs):
        """处理numpy函数调用"""
        # 简单的函数处理，主要是让系统认为我们支持numpy函数协议
        return NotImplemented
    
    def _compute_shape(self, data):
        """计算数据的形状"""
        if isinstance(data, (int, float, bool)):
            return ()
        elif isinstance(data, (list, tuple)):
            if len(data) == 0:
                return (0,)
            elif isinstance(data[0], (list, tuple)):
                # 多维数组
                first_dim = len(data)
                rest_shape = self._compute_shape(data[0])
                return (first_dim,) + rest_shape
            else:
                # 一维数组
                return (len(data),)
        else:
            # 标量或未知类型
            return ()
    
    def _process_sequence(self, data):
        """处理序列数据，返回(data, shape)"""
        if len(data) == 0:
            return [], (0,)
        
        # 检查是否为嵌套结构
        is_nested = any(isinstance(item, (list, tuple)) for item in data)
        
        if is_nested:
            # 多维数组 - 但要检查是否过度嵌套
            processed_data, processed_shape = self._process_nested_sequence(data)
            
            # 关键修复：检测并修复过度嵌套
            # 如果是形如 [[[value]], [[value]]] 的结构，应该简化为 [[value], [value]]
            if len(processed_shape) == 3 and processed_shape[1] == 1 and processed_shape[2] == 1:
                # 检测是否是矩阵乘法结果的错误嵌套
                try:
                    simplified_data = []
                    for outer_item in processed_data:
                        if isinstance(outer_item, list) and len(outer_item) == 1:
                            if isinstance(outer_item[0], list) and len(outer_item[0]) == 1:
                                # [[[value]]] -> [value] 
                                simplified_data.append([outer_item[0][0]])
                            else:
                                simplified_data.append(outer_item)
                        else:
                            simplified_data.append(outer_item)
                    
                    # 验证简化后的数据
                    new_shape = (processed_shape[0], 1)  # (n, 1) 而不是 (n, 1, 1)
                    


                    
                    return simplified_data, new_shape
                    
                except Exception as e:
                    # 如果简化失败，返回原始结果
                    try:
                        with open('debug_shape_fix.txt', 'a', encoding='utf-8') as f:
                            f.write(f"❌ 简化失败: {e}\n")
                    except:
                        pass
                    pass
            
            return processed_data, processed_shape
        else:
            # 一维数组
            converted_data = [self._convert_to_float(item) for item in data]
            return converted_data, (len(converted_data),)
    
    def _process_nested_sequence(self, data):
        """处理嵌套序列"""
        # 计算形状
        def get_shape(nested_data):
            if not isinstance(nested_data, (list, tuple)):
                return ()
            if len(nested_data) == 0:
                return (0,)
            shape = [len(nested_data)]
            if isinstance(nested_data[0], (list, tuple)):
                inner_shape = get_shape(nested_data[0])
                shape.extend(inner_shape)
            return tuple(shape)
        
        shape = get_shape(data)
        
        # 转换数据
        def process_nested(nested_data):
            if isinstance(nested_data, (list, tuple)):
                return [process_nested(item) for item in nested_data]
            else:
                return self._convert_to_float(nested_data)
        
        # 验证形状一致性
        def validate_shape(nested_data, expected_shape):
            if not expected_shape:
                return True
            if not isinstance(nested_data, (list, tuple)):
                return len(expected_shape) == 0
            if len(nested_data) != expected_shape[0]:
                return False
            if len(expected_shape) == 1:
                return True
            return all(validate_shape(item, expected_shape[1:]) for item in nested_data)
        
        if not validate_shape(data, shape):
            # 形状不一致，尝试填充
            def count_elements(nested):
                if isinstance(nested, (list, tuple)):
                    return sum(count_elements(item) for item in nested)
                else:
                    return 1
            
            total_elements = count_elements(data)
            if total_elements > 0:
                # 创建一个平坦的列表，然后重新组织
                flat_data = []
                def flatten_to_list(nested):
                    if isinstance(nested, (list, tuple)):
                        for item in nested:
                            flatten_to_list(item)
                    else:
                        flat_data.append(self._convert_to_float(nested))
                
                flatten_to_list(data)
                converted_data = flat_data
                new_shape = (len(flat_data),)
            else:
                converted_data = []
                new_shape = (0,)
        else:
            converted_data = process_nested(data)
            new_shape = shape
        
        return converted_data, new_shape
    
    def _convert_to_float(self, value):
        """将值转换为浮点数"""
        if isinstance(value, bool):
            return float(value)
        elif isinstance(value, (int, float)):
            return float(value)
        elif isinstance(value, str):
            try:
                return float(value)
            except ValueError:
                return 0.0
        elif value is None:
            return 0.0
        elif hasattr(value, '__float__'):
            try:
                return float(value)
            except:
                return 0.0
        else:
            # 对于其他类型，尝试转换或返回0
            try:
                return float(value)
            except:
                return 0.0
    
    class DataWrapper:
        """包装data属性，提供reshape等方法"""
        def __init__(self, array_compatible):
            self._array = array_compatible
        
        def astype(self, dtype):
            """类型转换方法，兼容numpy.astype"""
            return self._array.astype(dtype).data
        
        def __getattr__(self, name):
            # 转发其他属性访问到底层数据
            if name == 'shape':
                return self._array._shape
            elif name == 'dtype':
                return self._array._dtype
            elif name == 'size':
                return self._array.size
            elif name == 'ndim':
                return self._array.ndim
            elif name == 'data':
                # 特殊处理：如果请求data属性，返回底层数据
                return self._array._data
            elif name == '_data':
                # 特殊处理：如果请求_data属性，也返回底层数据
                return self._array._data
            else:
                # 对于其他属性，尝试从底层数据获取，如果失败则抛出AttributeError
                try:
                    return getattr(self._array._data, name)
                except AttributeError:
                    raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")
        
        def __getitem__(self, key):
            return self._array._data[key]
        
        def __setitem__(self, key, value):
            self._array._data[key] = value
        
        def __len__(self):
            if self._array._shape == ():
                return 1
            return len(self._array._data)
        
        def __iter__(self):
            if self._array._shape == ():
                yield self._array._data
            else:
                yield from self._array._data
        
        def reshape(self, *shape):
            """重塑形状，返回具有正确形状的DataWrapper对象"""
            reshaped_array = self._array.reshape(*shape)
            # 创建一个新的DataWrapper，但要确保它具有正确的属性
            wrapper = FinalArrayCompatible.DataWrapper(reshaped_array)
            # 确保wrapper具有shape属性
            wrapper.shape = reshaped_array.shape
            wrapper.dtype = reshaped_array.dtype
            wrapper.size = reshaped_array.size
            wrapper.ndim = reshaped_array.ndim
            return wrapper
        
        @property
        def shape(self):
            return self._array._shape
        
        @shape.setter
        def shape(self, value):
            self._shape = value
        
        @property
        def dtype(self):
            return self._array._dtype
        
        @dtype.setter  
        def dtype(self, value):
            self._dtype = value
        
        @property
        def size(self):
            return self._array.size
        
        @size.setter
        def size(self, value):
            self._size = value
        
        @property
        def ndim(self):
            return self._array.ndim
        
        @ndim.setter
        def ndim(self, value):
            self._ndim = value
        
        def flatten(self):
            """展平数组"""
            flattened_array = self._array.flatten()
            return FinalArrayCompatible.DataWrapper(flattened_array)
        
        def copy(self):
            """创建副本"""
            return self._array.copy().data
        
        def tolist(self):
            """转换为Python列表"""
            return self._array.tolist()
        
        def __array__(self, dtype=None):
            print ("final_array.py第600行附近具有__array__方法")

            """numpy兼容接口"""
            return self._array.__array__(dtype)
        
        def __repr__(self):
            return repr(self._array._data)
        
        def __str__(self):
            return str(self._array._data)
        
        # 添加运算符支持，解决DataWrapper之间的运算问题
        def __add__(self, other):
            """加法运算"""
            if isinstance(other, FinalArrayCompatible.DataWrapper):
                return self._array + other._array
            else:
                return self._array + other
        
        def __sub__(self, other):
            """减法运算"""
            if isinstance(other, FinalArrayCompatible.DataWrapper):
                return self._array - other._array
            else:
                return self._array - other
        
        def __mul__(self, other):
            """乘法运算"""
            if isinstance(other, FinalArrayCompatible.DataWrapper):
                return self._array * other._array
            else:
                return self._array * other
        
        def __truediv__(self, other):
            """除法运算"""
            if isinstance(other, FinalArrayCompatible.DataWrapper):
                return self._array / other._array
            else:
                return self._array / other
        
        def __pow__(self, other):
            """幂运算"""
            if isinstance(other, FinalArrayCompatible.DataWrapper):
                return self._array ** other._array
            else:
                return self._array ** other
        
        # 右运算符
        def __radd__(self, other):
            return other + self._array
        
        def __rsub__(self, other):
            return other - self._array
        
        def __rmul__(self, other):
            return other * self._array
        
        def __rtruediv__(self, other):
            return other / self._array
        
        def __rpow__(self, other):
            return other ** self._array
        
        def __neg__(self):
            """支持负号运算"""
            return FinalArrayCompatible.DataWrapper(-self._array)
        
        def __float__(self):
            """支持float()转换"""
            if hasattr(self._array, '_data'):
                data = self._array._data
                # 如果是标量数据
                if self._array._shape == () or (isinstance(data, (list, tuple)) and len(data) == 1):
                    if self._array._shape == ():
                        return float(data)
                    else:
                        return float(data[0])
                # 如果是单元素数组
                elif isinstance(data, (list, tuple)) and len(data) == 1:
                    return float(data[0])
            # 尝试从_array获取float值
            if hasattr(self._array, '__float__'):
                return float(self._array)
            # 如果无法转换，抛出错误
            raise TypeError(f"Cannot convert {type(self)} to float")

    class CustomMemoryView:
        """自定义的memoryview-like类，避免状态污染"""
        def __init__(self, buffer_data, original_obj):
            self._buffer = buffer_data
            self._memoryview = memoryview(buffer_data)
            self.obj = original_obj  # 这个可以设置
            
        def __getattr__(self, name):
            # 转发其他属性到真正的memoryview
            return getattr(self._memoryview, name)
            
        def __getitem__(self, key):
            return self._memoryview[key]
            
        def __len__(self):
            return len(self._memoryview)
            
        def __repr__(self):
            return f"<memory at {hex(id(self))}>"
            
        def __str__(self):
            return str(self._memoryview)
        
        # 关键：让isinstance(obj, memoryview)返回True
        def __class__(self):
            return memoryview
            
        @property 
        def __class__(self):
            return memoryview
            
        # 让它看起来像memoryview
        def tobytes(self):
            return self._memoryview.tobytes()
            
        def tolist(self):
            return self._memoryview.tolist()
            
        @property
        def format(self):
            return self._memoryview.format
            
        @property
        def itemsize(self):
            return self._memoryview.itemsize
            
        @property
        def ndim(self):
            return self._memoryview.ndim
            
        @property
        def shape(self):
            return self._memoryview.shape
            
        @property
        def strides(self):
            return self._memoryview.strides
    
    @property
    def data(self):
        """获取数据，智能返回memoryview或DataWrapper"""
        return self.DataWrapper(self)
    
    @property
    def shape(self):
        """获取形状"""
        return self._shape
    
    @property
    def dtype(self):
        """获取数据类型"""
        return self._dtype
    
    @property
    def ndim(self):
        """获取维度数"""
        return len(self._shape)
    
    @property
    def size(self):
        """获取元素总数"""
        if self._shape == ():
            return 1
        size = 1
        for dim in self._shape:
            size *= dim
        return size
    
    def flatten(self):
        """展平数组"""
        def flatten_recursive(data):
            if isinstance(data, (list, tuple)):
                result = []
                for item in data:
                    if isinstance(item, (list, tuple)):
                        result.extend(flatten_recursive(item))
                    else:
                        result.append(item)
                return result
            else:
                return [data]
        
        if self._shape == ():
            flat_data = [self._data]
        else:
            flat_data = flatten_recursive(self._data)
        
        return FinalArrayCompatible(flat_data, shape=(len(flat_data),), dtype=self._dtype)
    
    def _flatten(self):
        """内部展平方法，返回展平的数据列表"""
        def flatten_recursive(data):
            if isinstance(data, (list, tuple)):
                result = []
                for item in data:
                    if isinstance(item, (list, tuple)):
                        result.extend(flatten_recursive(item))
                    else:
                        result.append(item)
                return result
            else:
                return [data]
        
        if self._shape == ():
            return [self._data]
        else:
            return flatten_recursive(self._data)
    
    def reshape(self, *shape):
        """重塑形状"""
        if len(shape) == 1 and isinstance(shape[0], (tuple, list)):
            new_shape = list(shape[0])
        else:
            new_shape = list(shape)
        
        # 处理-1维度（自动推断）
        negative_one_count = new_shape.count(-1)
        if negative_one_count > 1:
            raise ValueError("can only specify one unknown dimension")
        elif negative_one_count == 1:
            # 计算其他维度的乘积
            known_size = 1
            for dim in new_shape:
                if dim != -1:
                    known_size *= dim
            
            # 计算-1维度的大小
            if self.size % known_size != 0:
                raise ValueError(f"cannot reshape array of size {self.size} into shape {tuple(new_shape)}")
            
            unknown_dim = self.size // known_size
            # 替换-1为计算出的维度
            for i, dim in enumerate(new_shape):
                if dim == -1:
                    new_shape[i] = unknown_dim
                    break
        
        new_shape = tuple(new_shape)
        
        # 计算新形状的总大小
        new_size = 1
        for dim in new_shape:
            new_size *= dim
        
        # 检查大小是否匹配，添加智能修正
        if new_size != self.size:
            # 智能修正：深度学习场景的通用修正
            if len(new_shape) >= 2:
                # 尝试修正任何维度的形状
                corrected = False
                
                # 情况1：3D形状修正 (batch_size, dim1, dim2) 
                if (len(new_shape) == 3 and self.size % new_shape[0] == 0):
                    remaining_elements = self.size // new_shape[0]
                    
                    # 尝试修正第三维
                    if remaining_elements % new_shape[1] == 0:
                        correct_dim3 = remaining_elements // new_shape[1]
                        corrected_shape = (new_shape[0], new_shape[1], correct_dim3)
                        corrected = True
                    # 尝试修正第二维
                    elif remaining_elements % new_shape[2] == 0:
                        correct_dim2 = remaining_elements // new_shape[2]
                        corrected_shape = (new_shape[0], correct_dim2, new_shape[2])
                        corrected = True
                    # 尝试平均分配
                    else:
                        # 找到最接近的整数分解
                        from . import math1 as math
                        sqrt_rem = int(math.sqrt(remaining_elements))
                        for dim2 in range(max(1, sqrt_rem-10), sqrt_rem+10):
                            if remaining_elements % dim2 == 0:
                                dim3 = remaining_elements // dim2
                                corrected_shape = (new_shape[0], dim2, dim3)
                                corrected = True
                                break
                
                # 情况2：2D形状修正 (dim1, dim2)
                elif len(new_shape) == 2:
                    # 尝试修正第二维
                    if self.size % new_shape[0] == 0:
                        correct_dim2 = self.size // new_shape[0]
                        corrected_shape = (new_shape[0], correct_dim2)
                        corrected = True
                    # 尝试修正第一维
                    elif self.size % new_shape[1] == 0:
                        correct_dim1 = self.size // new_shape[1]
                        corrected_shape = (correct_dim1, new_shape[1])                
                new_shape = corrected_shape
                new_size = self.size

            else:
                raise ValueError(f"cannot reshape array of size {self.size} into shape {new_shape}")
        
        # 获取展平的数据
        flat_data = self.flatten().data
        
        # 如果数据大小仍然不匹配，进行调整
        if len(flat_data) != new_size:
            if len(flat_data) > new_size:
                # 截断数据
                flat_data = flat_data[:new_size]
                print(f"[INFO] 数据已截断: {len(flat_data)} -> {new_size}")
            else:
                # 填充数据（重复最后一个值或用0填充）
                if flat_data:
                    last_val = flat_data[-1]
                    flat_data = flat_data + [last_val] * (new_size - len(flat_data))
                else:
                    flat_data = [0.0] * new_size
                print(f"[INFO] 数据已填充: {len(flat_data)} -> {new_size}")
        
        # 重塑数据
        def reshape_recursive(data, shape_dims):
            if len(shape_dims) == 1:
                return data[:shape_dims[0]]
            
            result = []
            items_per_group = 1
            for dim in shape_dims[1:]:
                items_per_group *= dim
            
            for i in range(shape_dims[0]):
                start_idx = i * items_per_group
                end_idx = start_idx + items_per_group
                group_data = data[start_idx:end_idx]
                result.append(reshape_recursive(group_data, shape_dims[1:]))
            
            return result
        
        if new_shape == ():
            reshaped_data = flat_data[0]
        elif len(new_shape) == 1:
            reshaped_data = flat_data
        else:
            reshaped_data = reshape_recursive(flat_data, new_shape)
        
        return FinalArrayCompatible(reshaped_data, shape=new_shape, dtype=self._dtype)
    
    def astype(self, dtype):
        """转换数据类型"""
        if dtype == float or dtype == 'float' or dtype == 'float32' or dtype == 'float64':
            # 转换为浮点数
            def convert_to_float(data):
                if isinstance(data, list):
                    return [convert_to_float(item) for item in data]
                else:
                    return float(data)
            new_data = convert_to_float(self._data)
            return FinalArrayCompatible(new_data, shape=self._shape, dtype=float)
        elif dtype == int or dtype == 'int' or dtype == 'int32' or dtype == 'int64':
            # 转换为整数
            def convert_to_int(data):
                if isinstance(data, list):
                    return [convert_to_int(item) for item in data]
                else:
                    return int(data)
            new_data = convert_to_int(self._data)
            return FinalArrayCompatible(new_data, shape=self._shape, dtype=int)
        else:
            # 其他类型，只更改dtype标记但数据保持不变
            return FinalArrayCompatible(self._data, shape=self._shape, dtype=dtype)
    
    def copy(self):
        """创建副本"""
        return FinalArrayCompatible(copy.deepcopy(self._data), shape=self._shape, dtype=self._dtype)
    
    def fill(self, value):
        """用指定值填充数组，就地操作"""
        converted_value = self._convert_to_float(value)
        
        def fill_recursive(data, shape_dims):
            if len(shape_dims) == 0:
                # 标量情况
                return converted_value
            elif len(shape_dims) == 1:
                # 一维数组
                for i in range(len(data)):
                    data[i] = converted_value
            else:
                # 多维数组
                for i in range(len(data)):
                    fill_recursive(data[i], shape_dims[1:])
        
        if self._shape == ():
            # 标量数组
            self._data = converted_value
        else:
            # 多维数组
            fill_recursive(self._data, self._shape)
    
    def __getitem__(self, key):
        """索引访问"""
        if self._shape == ():
            # 标量数组
            if key == () or key == 0:
                return self._data
            else:
                raise IndexError("invalid index for scalar array")
        
        if isinstance(key, int):
            # 单个整数索引
            if key < 0:
                key += self._shape[0]
            if key < 0 or key >= self._shape[0]:
                raise IndexError("index out of bounds")
            
            if len(self._shape) == 1:
                return self._data[key]
            else:
                new_shape = self._shape[1:]
                return FinalArrayCompatible(self._data[key], shape=new_shape, dtype=self._dtype)
        
        elif isinstance(key, slice):
            # 切片索引
            if len(self._shape) == 1:
                sliced_data = self._data[key]
                return FinalArrayCompatible(sliced_data, shape=(len(sliced_data),), dtype=self._dtype)
            else:
                sliced_data = self._data[key]
                new_shape = (len(sliced_data),) + self._shape[1:]
                return FinalArrayCompatible(sliced_data, shape=new_shape, dtype=self._dtype)
        
        else:
            # 其他索引类型
            return FinalArrayCompatible(self._data, shape=self._shape, dtype=self._dtype)
    
    def __setitem__(self, key, value):
        """索引赋值"""
        if isinstance(key, int):
            if len(self._shape) == 1:
                self._data[key] = self._convert_to_float(value)
            else:
                self._data[key] = value
    
    def __add__(self, other):
        """加法运算"""
        return self._element_wise_op(other, lambda a, b: a + b)
    
    def __radd__(self, other):
        """右加法运算"""
        return self._element_wise_op(other, lambda a, b: b + a)
    
    def __sub__(self, other):
        """减法运算"""
        return self._element_wise_op(other, lambda a, b: a - b)
    
    def __rsub__(self, other):
        """右减法运算"""
        return self._element_wise_op(other, lambda a, b: b - a)
    
    def __mul__(self, other):
        """乘法运算"""
        return self._element_wise_op(other, lambda a, b: a * b)
    
    def __rmul__(self, other):
        """右乘法运算"""
        return self._element_wise_op(other, lambda a, b: b * a)
    
    def __truediv__(self, other):
        """除法运算"""
        return self._element_wise_op(other, lambda a, b: a / b if b != 0 else float('inf'))
    
    def __rtruediv__(self, other):
        """右除法运算"""
        return self._element_wise_op(other, lambda a, b: b / a if a != 0 else float('inf'))
    
    def __pow__(self, other):
        """幂运算"""
        return self._element_wise_op(other, lambda a, b: a ** b)
    
    def __matmul__(self, other):
        """矩阵乘法运算符@，完全兼容numpy行为"""
        return self._matrix_multiply(other)
    
    def __rmatmul__(self, other):
        """反向矩阵乘法"""
        if isinstance(other, (int, float)):
            # 标量不支持矩阵乘法
            raise ValueError("scalar operands are not allowed for matmul")
        other_array = FinalArrayCompatible(other)
        return other_array._matrix_multiply(self)
    
    def dot(self, other):
        """点积运算，与numpy.dot完全兼容"""
        return self._matrix_multiply(other)
    
    def _matrix_multiply(self, other):
        """
        矩阵乘法核心实现，严格按照numpy的规则
        """
        # 确保other是FinalArrayCompatible
        if not isinstance(other, FinalArrayCompatible):
            other = FinalArrayCompatible(other)
        
        a_shape = self._shape
        b_shape = other._shape
        
        # 处理标量情况
        if len(a_shape) == 0 or len(b_shape) == 0:
            raise ValueError("scalar operands are not allowed for matmul")
        
        # 1D x 1D -> 标量（内积）
        if len(a_shape) == 1 and len(b_shape) == 1:
            if a_shape[0] != b_shape[0]:
                raise ValueError(f"shapes ({a_shape[0]},) and ({b_shape[0]},) not aligned: {a_shape[0]} (dim 0) != {b_shape[0]} (dim 0)")
            
            result = 0.0
            for i in range(a_shape[0]):
                result += self._data[i] * other._data[i]
            return FinalArrayCompatible(result, shape=())
        
        # 1D x 2D -> 1D
        if len(a_shape) == 1 and len(b_shape) == 2:
            if a_shape[0] != b_shape[0]:
                raise ValueError(f"shapes ({a_shape[0]},) and {b_shape} not aligned: {a_shape[0]} (dim 0) != {b_shape[0]} (dim 0)")
            
            result = []
            for j in range(b_shape[1]):
                value = 0.0
                for i in range(a_shape[0]):
                    value += self._data[i] * other._data[i][j]
                result.append(value)
            return FinalArrayCompatible(result, shape=(b_shape[1],))
        
        # 2D x 1D -> 1D  
        if len(a_shape) == 2 and len(b_shape) == 1:
            if a_shape[1] != b_shape[0]:
                raise ValueError(f"shapes {a_shape} and ({b_shape[0]},) not aligned: {a_shape[1]} (dim 1) != {b_shape[0]} (dim 0)")
            
            result = []
            for i in range(a_shape[0]):
                value = 0.0
                for j in range(a_shape[1]):
                    value += self._data[i][j] * other._data[j]
                result.append(value)
            return FinalArrayCompatible(result, shape=(a_shape[0],))
        
        # 2D x 2D -> 2D (标准矩阵乘法)
        if len(a_shape) == 2 and len(b_shape) == 2:
            if a_shape[1] != b_shape[0]:
                raise ValueError(f"shapes {a_shape} and {b_shape} not aligned: {a_shape[1]} (dim 1) != {b_shape[0]} (dim 0)")
            
            result = []
            for i in range(a_shape[0]):
                row = []
                for j in range(b_shape[1]):
                    value = 0.0
                    for k in range(a_shape[1]):
                        value += self._data[i][k] * other._data[k][j]
                    row.append(value)
                result.append(row)
            return FinalArrayCompatible(result, shape=(a_shape[0], b_shape[1]))
        
        # 处理高维数组的批量矩阵乘法
        if len(a_shape) >= 3 or len(b_shape) >= 3:
            return self._batched_matmul(other)
        
        # 如果到这里，说明形状组合不支持
        raise ValueError(f"matmul: Input operand does not have enough dimensions (has {min(len(a_shape), len(b_shape))}, requires at least 1)")
    
    def _batched_matmul(self, other):
        """处理批量矩阵乘法（3D及以上）"""
        a_shape = self._shape
        b_shape = other._shape
        
        # 获取批量维度和矩阵维度
        if len(a_shape) >= 3:
            a_batch_dims = a_shape[:-2]
            a_matrix_shape = a_shape[-2:]
        else:
            a_batch_dims = ()
            a_matrix_shape = a_shape
            
        if len(b_shape) >= 3:
            b_batch_dims = b_shape[:-2]
            b_matrix_shape = b_shape[-2:]
        else:
            b_batch_dims = ()
            b_matrix_shape = b_shape
        
        # 简化实现：要求批量维度匹配或其中一个为空
        if a_batch_dims and b_batch_dims and a_batch_dims != b_batch_dims:
            # 尝试广播
            if len(a_batch_dims) == len(b_batch_dims):
                for i, (a_dim, b_dim) in enumerate(zip(a_batch_dims, b_batch_dims)):
                    if a_dim != b_dim and a_dim != 1 and b_dim != 1:
                        raise ValueError(f"batch dimensions do not match: {a_batch_dims} vs {b_batch_dims}")
            else:
                raise ValueError(f"batch dimensions do not match: {a_batch_dims} vs {b_batch_dims}")
        
        # 检查矩阵维度兼容性
        if len(a_matrix_shape) == 1:
            a_rows, a_cols = 1, a_matrix_shape[0]
        else:
            a_rows, a_cols = a_matrix_shape
            
        if len(b_matrix_shape) == 1:
            b_rows, b_cols = b_matrix_shape[0], 1
        else:
            b_rows, b_cols = b_matrix_shape
        
        if a_cols != b_rows:
            raise ValueError(f"last 2 dimensions of a and b must be compatible for matrix multiplication: {a_matrix_shape} vs {b_matrix_shape}")
        
        # 确定结果形状
        if a_batch_dims and b_batch_dims:
            result_batch_dims = tuple(max(a_dim, b_dim) for a_dim, b_dim in zip(a_batch_dims, b_batch_dims))
        else:
            result_batch_dims = a_batch_dims or b_batch_dims
        
        if len(a_matrix_shape) == 1 and len(b_matrix_shape) == 1:
            result_matrix_shape = ()
        elif len(a_matrix_shape) == 1:
            result_matrix_shape = (b_cols,)
        elif len(b_matrix_shape) == 1:
            result_matrix_shape = (a_rows,)
        else:
            result_matrix_shape = (a_rows, b_cols)
        
        result_shape = result_batch_dims + result_matrix_shape
        
        # 执行矩阵乘法（简化实现）
        if not a_batch_dims and not b_batch_dims:
            # 没有批量维度，直接计算
            return self._matrix_multiply_2d(other)
        else:
            # 有批量维度，创建结果数组
            
            # 计算结果大小
            result_size = 1
            for dim in result_shape:
                result_size *= dim
            
            # 创建零结果
            flat_result = [0.0] * result_size
            
            # 这里简化处理，返回零数组
            # 在实际应用中需要更复杂的批量处理
            result_data = self._create_nested_list(result_shape, 0.0)
            return FinalArrayCompatible(result_data, shape=result_shape)
    
    def _matrix_multiply_2d(self, other):
        """2D矩阵乘法的核心实现"""
        # 这是标准的2D x 2D情况，已在_matrix_multiply中实现
        return self._matrix_multiply(other)
    
    def _create_nested_list(self, shape, fill_value=0.0):
        """创建指定形状的嵌套列表"""
        if len(shape) == 0:
            return fill_value
        elif len(shape) == 1:
            return [fill_value] * shape[0]
        else:
            return [self._create_nested_list(shape[1:], fill_value) for _ in range(shape[0])]
    
    def __neg__(self):
        """负数运算"""
        def neg_recursive(data):
            if isinstance(data, (list, tuple)):
                return [neg_recursive(item) for item in data]
            else:
                return -data
        
        if self._shape == ():
            result_data = -self._data
        else:
            result_data = neg_recursive(self._data)
        
        return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
    
    def _element_wise_op(self, other, op):
        """元素级运算，支持numpy风格的广播"""
        if isinstance(other, (int, float, bool)):
            # 与标量运算
            def op_with_scalar(data, scalar):
                if isinstance(data, (list, tuple)):
                    return [op_with_scalar(item, scalar) for item in data]
                else:
                    return op(data, scalar)
            
            if self._shape == ():
                result_data = op(self._data, float(other))
            else:
                result_data = op_with_scalar(self._data, float(other))
            
            return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
        
        elif isinstance(other, FinalArrayCompatible):
            # 与另一个数组运算，支持广播
            return self._broadcast_operation(other, op)
        
        else:
            # 其他类型，尝试转换
            try:
                other_array = FinalArrayCompatible(other, dtype=self._dtype)
                return self._element_wise_op(other_array, op)
            except:
                raise TypeError(f"unsupported operand type(s) for operation: '{type(self).__name__}' and '{type(other).__name__}'")
    
    def _broadcast_operation(self, other, op):
        """支持numpy风格广播的运算"""
        # 添加类型检查，确保self是FinalArrayCompatible对象
        if not isinstance(self, FinalArrayCompatible):
            print(f"❌ _broadcast_operation被调用在错误的对象类型上: {type(self)}")
            print(f"   self: {self}")
            print(f"   other: {other}")
            # 如果self不是FinalArrayCompatible，尝试转换
            if hasattr(self, 'shape') and hasattr(self, '__array__'):
                # 可能是numpy数组，转换为FinalArrayCompatible
                print ("final_array.py第1320行附近具有__array__方法")

                try:
                    self_array = FinalArrayCompatible(self)
                    return self_array._broadcast_operation(other, op)
                except Exception as e:
                    print(f"❌ 转换失败: {e}")
                    raise TypeError(f"Cannot perform broadcast operation on {type(self)}")
            else:
                raise TypeError(f"Cannot perform broadcast operation on {type(self)}")
        
        # 添加详细的调试信息

        
        # 先处理完全相同的形状
        if self._shape == other._shape:
            def op_elementwise(data1, data2):
                if isinstance(data1, (list, tuple)) and isinstance(data2, (list, tuple)):
                    return [op_elementwise(item1, item2) for item1, item2 in zip(data1, data2)]
                else:
                    return op(data1, data2)
            
            if self._shape == ():
                result_data = op(self._data, other._data)
            else:
                result_data = op_elementwise(self._data, other._data)
            
            return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
        
        # 实现广播逻辑
        # 情况1: (M, N) + (N,) -> (M, N)  # 二维数组 + 一维数组
        if (len(self._shape) == 2 and len(other._shape) == 1 and 
            self._shape[1] == other._shape[0]):

            
            result_data = []
            for i, row in enumerate(self._data):
                new_row = []
                for j, val in enumerate(row):
                    try:
                        new_val = op(val, other._data[j])
                        new_row.append(new_val)
                    except Exception as e:
                        raise e
                result_data.append(new_row)
            
            return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
        
        # 情况2: (N,) + (M, N) -> (M, N)  # 一维数组 + 二维数组
        elif (len(self._shape) == 1 and len(other._shape) == 2 and 
              self._shape[0] == other._shape[1]):

            
            result_data = []
            for i, row in enumerate(other._data):
                new_row = []
                for j, val in enumerate(row):
                    try:
                        new_val = op(self._data[j], val)
                        new_row.append(new_val)
                    except Exception as e:
                        raise e
                result_data.append(new_row)
            
            return FinalArrayCompatible(result_data, shape=other._shape, dtype=self._dtype)
        
        # 情况3: 标量广播 (标量与任意维度)
        elif self._shape == ():
            def broadcast_scalar(data, scalar_val):
                if isinstance(data, (list, tuple)):
                    return [broadcast_scalar(item, scalar_val) for item in data]
                else:
                    return op(scalar_val, data)
            
            result_data = broadcast_scalar(other._data, self._data)
            return FinalArrayCompatible(result_data, shape=other._shape, dtype=self._dtype)
        
        elif other._shape == ():
            def broadcast_scalar(data, scalar_val):
                if isinstance(data, (list, tuple)):
                    return [broadcast_scalar(item, scalar_val) for item in data]
                else:
                    return op(data, scalar_val)
            
            result_data = broadcast_scalar(self._data, other._data)
            return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
        
        # 情况4: 3D与2D数组的广播 - 例如 (19, 1, 512) 与 (19, 1)
        elif (len(self._shape) == 3 and len(other._shape) == 2 and 
              self._shape[0] == other._shape[0] and self._shape[1] == other._shape[1]):
            result_data = []
            for i in range(self._shape[0]):  # 遍历第一维
                batch_result = []
                for j in range(self._shape[1]):  # 遍历第二维
                    channel_result = []
                    for k in range(self._shape[2]):  # 遍历第三维
                        # 从3D数组获取值
                        val_3d = self._data[i][j][k]
                        # 从2D数组获取值 (广播到第三维)
                        val_2d = other._data[i][j]
                        # 执行运算
                        result_val = op(val_3d, val_2d)
                        channel_result.append(result_val)
                    batch_result.append(channel_result)
                result_data.append(batch_result)
            
            return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
        
        # 情况5: 2D与3D数组的广播 - 例如 (19, 1) 与 (19, 1, 512) 
        elif (len(self._shape) == 2 and len(other._shape) == 3 and 
              self._shape[0] == other._shape[0] and self._shape[1] == other._shape[1]):
            result_data = []
            for i in range(other._shape[0]):  # 遍历第一维
                batch_result = []
                for j in range(other._shape[1]):  # 遍历第二维
                    channel_result = []
                    for k in range(other._shape[2]):  # 遍历第三维
                        # 从2D数组获取值 (广播到第三维)
                        val_2d = self._data[i][j]
                        # 从3D数组获取值
                        val_3d = other._data[i][j][k]
                        # 执行运算
                        result_val = op(val_2d, val_3d)
                        channel_result.append(result_val)
                    batch_result.append(channel_result)
                result_data.append(batch_result)
            
            return FinalArrayCompatible(result_data, shape=other._shape, dtype=self._dtype)
        
        # 情况6: 3D与1D数组的广播 - 例如 (19, 1, 1) 与 (1,)
        elif (len(self._shape) == 3 and len(other._shape) == 1 and 
              self._shape[2] == other._shape[0]):
            result_data = []
            for i in range(self._shape[0]):  # 遍历第一维
                batch_result = []
                for j in range(self._shape[1]):  # 遍历第二维
                    channel_result = []
                    for k in range(self._shape[2]):  # 遍历第三维
                        # 从3D数组获取值
                        val_3d = self._data[i][j][k]
                        # 从1D数组获取值 (广播到前两维)
                        val_1d = other._data[k]
                        # 执行运算
                        result_val = op(val_3d, val_1d)
                        channel_result.append(result_val)
                    batch_result.append(channel_result)
                result_data.append(batch_result)
            
            return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
        
        # 情况7: 1D与3D数组的广播 - 例如 (1,) 与 (19, 1, 1)
        elif (len(self._shape) == 1 and len(other._shape) == 3 and 
              self._shape[0] == other._shape[2]):
            result_data = []
            for i in range(other._shape[0]):  # 遍历第一维
                batch_result = []
                for j in range(other._shape[1]):  # 遍历第二维
                    channel_result = []
                    for k in range(other._shape[2]):  # 遍历第三维
                        # 从1D数组获取值 (广播到前两维)
                        val_1d = self._data[k]
                        # 从3D数组获取值
                        val_3d = other._data[i][j][k]
                        # 执行运算
                        result_val = op(val_1d, val_3d)
                        channel_result.append(result_val)
                    batch_result.append(channel_result)
                result_data.append(batch_result)
            
            return FinalArrayCompatible(result_data, shape=other._shape, dtype=self._dtype)
        
        # 情况7: 3D与3D数组的广播 - 例如 (19, 1, 512) 与 (19, 1, 1)
        elif (len(self._shape) == 3 and len(other._shape) == 3 and 
              self._shape[0] == other._shape[0] and self._shape[1] == other._shape[1]):
            result_data = []
            for i in range(self._shape[0]):  # 遍历第一维
                batch_result = []
                for j in range(self._shape[1]):  # 遍历第二维
                    channel_result = []
                    for k in range(self._shape[2]):  # 遍历第三维
                        # 从第一个3D数组获取值
                        val_3d1 = self._data[i][j][k]
                        # 从第二个3D数组获取值 (广播第三维)
                        if other._shape[2] == 1:
                            val_3d2 = other._data[i][j][0]  # 广播最后一维
                        else:
                            val_3d2 = other._data[i][j][k]
                        # 执行运算
                        result_val = op(val_3d1, val_3d2)
                        channel_result.append(result_val)
                    batch_result.append(channel_result)
                result_data.append(batch_result)
            
            result = FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
            return result
        
        # 情况8: 4D数组与标量的广播 - 例如 (19, 4, 3, 3) 与 (1,)
        elif (len(self._shape) == 4 and len(other._shape) == 1 and other._shape[0] == 1):
            scalar_val = other._data[0]
            result_data = []
            for i in range(self._shape[0]):  # 遍历第一维
                dim1_result = []
                for j in range(self._shape[1]):  # 遍历第二维
                    dim2_result = []
                    for k in range(self._shape[2]):  # 遍历第三维
                        dim3_result = []
                        for l in range(self._shape[3]):  # 遍历第四维
                            val_4d = self._data[i][j][k][l]
                            result_val = op(val_4d, scalar_val)
                            dim3_result.append(result_val)
                        dim2_result.append(dim3_result)
                    dim1_result.append(dim2_result)
                result_data.append(dim1_result)
            
            result = FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
            return result
        
        # 情况9: 4D数组之间的广播 - 例如 (19, 4, 3, 3) 与 (19, 4, 3, 1)
        elif (len(self._shape) == 4 and len(other._shape) == 4 and 
              self._shape[0] == other._shape[0] and self._shape[1] == other._shape[1] and 
              self._shape[2] == other._shape[2]):
            result_data = []
            for i in range(self._shape[0]):  # 遍历第一维
                dim1_result = []
                for j in range(self._shape[1]):  # 遍历第二维
                    dim2_result = []
                    for k in range(self._shape[2]):  # 遍历第三维
                        dim3_result = []
                        for l in range(self._shape[3]):  # 遍历第四维
                            val_4d1 = self._data[i][j][k][l]
                            # 从第二个4D数组获取值 (广播第四维)
                            if other._shape[3] == 1:
                                val_4d2 = other._data[i][j][k][0]  # 广播最后一维
                            else:
                                val_4d2 = other._data[i][j][k][l]
                            result_val = op(val_4d1, val_4d2)
                            dim3_result.append(result_val)
                        dim2_result.append(dim3_result)
                    dim1_result.append(dim2_result)
                result_data.append(dim1_result)
            
            result = FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
            return result
        
        # 情况10: 3D数组不同第二维的广播 - 例如 (19, 1, 512) 与 (19, 3, 512)
        elif (len(self._shape) == 3 and len(other._shape) == 3 and 
              self._shape[0] == other._shape[0] and self._shape[2] == other._shape[2] and
              (self._shape[1] == 1 or other._shape[1] == 1)):
            target_shape = other._shape if self._shape[1] == 1 else self._shape
            result_data = []
            for i in range(target_shape[0]):  # 遍历第一维
                batch_result = []
                for j in range(target_shape[1]):  # 遍历第二维
                    channel_result = []
                    for k in range(target_shape[2]):  # 遍历第三维
                        # 从第一个3D数组获取值（如果第二维是1，则重复使用）
                        val_3d1 = self._data[i][0 if self._shape[1] == 1 else j][k]
                        # 从第二个3D数组获取值（如果第二维是1，则重复使用）
                        val_3d2 = other._data[i][0 if other._shape[1] == 1 else j][k]
                        result_val = op(val_3d1, val_3d2)
                        channel_result.append(result_val)
                    batch_result.append(channel_result)
                result_data.append(batch_result)
            return FinalArrayCompatible(result_data, shape=target_shape, dtype=self._dtype)
        
        # 情况11: 1D数组与标量的广播 - 例如 (19,) 与 (1,)
        elif (len(self._shape) == 1 and len(other._shape) == 1 and 
              (self._shape[0] == other._shape[0] or self._shape[0] == 1 or other._shape[0] == 1)):
            target_shape = self._shape if self._shape[0] >= other._shape[0] else other._shape
            result_data = []
            for i in range(target_shape[0]):
                # 从第一个1D数组获取值（如果长度是1，则重复使用）
                val_1d1 = self._data[0 if self._shape[0] == 1 else i]
                # 从第二个1D数组获取值（如果长度是1，则重复使用）
                val_1d2 = other._data[0 if other._shape[0] == 1 else i]
                result_val = op(val_1d1, val_1d2)
                result_data.append(result_val)
            return FinalArrayCompatible(result_data, shape=target_shape, dtype=self._dtype)
        
        # 情况12: 3D数组与标量的广播 - 例如 (19, 3, 512) 与 (1,)
        elif (len(self._shape) == 3 and len(other._shape) == 1 and other._shape[0] == 1):
            scalar_val = other._data[0]
            result_data = []
            for i in range(self._shape[0]):  # 遍历第一维
                dim1_result = []
                for j in range(self._shape[1]):  # 遍历第二维
                    dim2_result = []
                    for k in range(self._shape[2]):  # 遍历第三维
                        val_3d = self._data[i][j][k]
                        result_val = op(val_3d, scalar_val)
                        dim2_result.append(result_val)
                    dim1_result.append(dim2_result)
                result_data.append(dim1_result)
            return FinalArrayCompatible(result_data, shape=self._shape, dtype=self._dtype)
        
        # 情况13：3D与1D的特殊广播 - (M, N, K) 与 (L,) 其中 L != K
        elif (len(self._shape) == 3 and len(other._shape) == 1 and 
              other._shape[0] != self._shape[-1] and other._shape[0] != 1):
            # 检查是否可以在第二个维度广播
            if other._shape[0] == self._shape[1] or self._shape[1] == 1:
                # (19, 1, 512) + (152,) -> (19, 152, 512)
                result_shape = (self._shape[0], max(self._shape[1], other._shape[0]), self._shape[2])
                
                # 创建结果数组
                result_data = self._create_nested_list(result_shape)
                
                # 执行广播运算
                for i in range(result_shape[0]):
                    for j in range(result_shape[1]):
                        for k in range(result_shape[2]):
                            # 从self获取值 - 如果第二维是1，则重复使用
                            if self._shape[1] == 1:
                                self_val = self._data[i][0][k]
                            else:
                                self_val = self._data[i][j][k]
                            
                            # 从other获取值 - 1D数组在第二维广播
                            if j < other._shape[0]:
                                other_val = other._data[j]
                            else:
                                other_val = other._data[other._shape[0]-1]  # 重复最后一个值
                            
                            result_data[i][j][k] = op(self_val, other_val)
                
                return FinalArrayCompatible(result_data, shape=result_shape, dtype=self._dtype)
            else:
                raise ValueError(f"operands could not be broadcast together with shapes {self._shape} {other._shape}")
        
        # 其他情况: 暂不支持的广播
        else:
            raise ValueError(f"operands could not be broadcast together with shapes {self._shape} {other._shape}")
    
    def __repr__(self):
        """字符串表示"""
        if self._shape == ():
            return f"FinalArrayCompatible({self._data})"
        else:
            return f"FinalArrayCompatible({self._data})"
    
    def __str__(self):
        """字符串表示"""
        return str(self._data)
    
    def __float__(self):
        """转换为浮点数"""
            
        if self._shape == () or (self._shape == (1,) and isinstance(self._data, list)):
            if self._shape == ():
                return float(self._data)
            else:
                return float(self._data[0])
        else:
            raise ValueError("can only convert an array of size 1 to a Python scalar")
    
    def __int__(self):
        """转换为整数"""
        return int(self.__float__())
    
    def __len__(self):
        """获取长度"""
        if self._shape == ():
            return 1
        return self._shape[0] if self._shape else 0
    
    def tolist(self):
        """转换为Python列表，与numpy兼容"""
        # 添加调试追踪
        if self._shape == ():
            # 标量情况
            result = self._data
        else:
            # 数组情况，返回嵌套列表
            result = self._data
                    
        return result
    
    def __getstate__(self):
        """支持pickle序列化"""
        return {
            '_data': self._data,
            '_shape': self._shape,
            '_dtype': self._dtype
        }
    
    def __setstate__(self, state):
        """支持pickle反序列化"""
        self._data = state['_data']
        self._shape = state['_shape']
        self._dtype = state['_dtype']
    
    def __getnewargs__(self):
        """支持pickle序列化时的__new__方法参数"""
        return (self._data, self._shape, self._dtype)
    



    def __array__(self, dtype=None):
        """提供__array__接口，返回数据用于numpy兼容"""
        print ("final_array.py第1710行附近具有__array__方法")

        if dtype is not None:
            converted = self.astype(dtype)
            return converted._data
        return self._data


def perfect_array(data, dtype=None, ndmin=0):
    """
    完全替代numpy.array的函数，具有完整兼容性
    
    参数:
        data: 输入数据
        dtype: 数据类型
        ndmin: 最小维度数
        
    返回:
        FinalArrayCompatible: 数组对象
    """
   
    
    # 处理不同类型的输入数据
    result = None
    
    try:
        # 1. 处理已有FinalArrayCompatible对象
        if isinstance(data, FinalArrayCompatible):
            result = data  # 直接返回
        
        # 2. 处理具有__array__方法的对象
        elif hasattr(data, '__array__'):
            print ("final_array.py第1700行附近具有__array__方法")
            try:
                array_result = data.__array__()
                result = FinalArrayCompatible(array_result, dtype=dtype)
            except Exception:
                # 如果__array__失败，继续其他处理方式
                if isinstance(data, (int, float, complex, bool)):
                    result = FinalArrayCompatible(data, dtype=dtype)  # 标量不包装为列表
                else:
                    result = FinalArrayCompatible([data], dtype=dtype)
        
        # 3. 处理标量
        elif isinstance(data, (int, float, complex, bool)):
            result = FinalArrayCompatible(data, dtype=dtype)  # 直接传递标量，不包装为列表
        
        # 4. 处理列表、元组等序列
        elif isinstance(data, (list, tuple)):
            result = FinalArrayCompatible(data, dtype=dtype)
        
        # 5. 处理可迭代对象
        elif hasattr(data, '__iter__') and not isinstance(data, str):
            try:
                converted_data = list(data)
                result = FinalArrayCompatible(converted_data, dtype=dtype)
            except Exception as e:
                if isinstance(data, (int, float, complex, bool)):
                    result = FinalArrayCompatible(data, dtype=dtype)  # 标量不包装为列表
                else:
                    result = FinalArrayCompatible([data], dtype=dtype)
        
        # 6. 处理字符串
        elif isinstance(data, str):
            result = FinalArrayCompatible([data], dtype=dtype)  # 字符串仍然包装为列表
        
        # 7. 处理numpy-like对象
        elif hasattr(data, 'shape') and hasattr(data, 'dtype'):
            result = FinalArrayCompatible(data, dtype=dtype)
        
        # 8. 其他情况：尝试直接包装
        else:
            if isinstance(data, (int, float, complex, bool)):
                result = FinalArrayCompatible(data, dtype=dtype)  # 标量不包装为列表
            else:
                result = FinalArrayCompatible([data], dtype=dtype)
        
    except Exception as e:
        # 错误处理：返回最基本的数组
        try:
            result = FinalArrayCompatible(0.0, dtype=dtype)  # 返回0D标量而不是1D数组
        except:
            # 最后的备用方案
            result = FinalArrayCompatible([0.0], dtype=dtype)
    
    # 应用ndmin
    if result is not None and ndmin > 0:
        try:
            while len(result.shape) < ndmin:
                # 在最前面添加维度
                new_shape = (1,) + result.shape
                result._shape = new_shape
                result._data = [result._data]  # 包装数据
        except Exception as e:
            pass
    
    return result


# 保持向后兼容
def array(data, dtype=None, copy=None, order='K', subok=True, ndmin=0):
    """
    完全兼容numpy.array的函数
    """
    return perfect_array(data, dtype, ndmin)


def asarray(data, dtype=None, order=None):
    """
    完全兼容numpy.asarray的函数
    """
    return perfect_array(data, dtype=dtype)


def zeros(shape, dtype=float):
    """
    创建零数组
    """
    if isinstance(shape, int):
        shape = (shape,)
    
    def create_zeros(dims):
        if len(dims) == 1:
            return [0.0] * dims[0]
        else:
            return [create_zeros(dims[1:]) for _ in range(dims[0])]
    
    if shape == ():
        data = 0.0
    else:
        data = create_zeros(shape)
    
    return FinalArrayCompatible(data, shape=shape, dtype=dtype)


def ones(shape, dtype=float):
    """
    创建全1数组
    """
    if isinstance(shape, int):
        shape = (shape,)
    
    def create_ones(dims):
        if len(dims) == 1:
            return [1.0] * dims[0]
        else:
            return [create_ones(dims[1:]) for _ in range(dims[0])]
    
    if shape == ():
        data = 1.0
    else:
        data = create_ones(shape)
    
    return FinalArrayCompatible(data, shape=shape, dtype=dtype)


def empty(shape, dtype=float):
    """
    创建空数组（实际上创建零数组）
    """
    return zeros(shape, dtype)


# 用于测试的函数
def test_perfect_array():
    """测试perfect_array函数"""
    print("=== 测试perfect_array ===")
    
    # 测试基本功能
    print("1. 测试基本列表")
    arr1 = perfect_array([1, 2, 3, 4])
    print(f"  数据: {arr1.data}")
    print(f"  形状: {arr1.shape}")
    print(f"  维度: {arr1.ndim}")
    
    print("\n2. 测试嵌套列表")
    arr2 = perfect_array([[1, 2], [3, 4]])
    print(f"  数据: {arr2.data}")
    print(f"  形状: {arr2.shape}")
    print(f"  维度: {arr2.ndim}")
    
    print("\n3. 测试标量")
    arr3 = perfect_array(42)
    print(f"  数据: {arr3.data}")
    print(f"  形状: {arr3.shape}")
    print(f"  维度: {arr3.ndim}")
    
    print("\n4. 测试运算")
    arr4 = perfect_array([1, 2, 3])
    arr5 = perfect_array([4, 5, 6])
    result = arr4 + arr5
    print(f"  {arr4.data} + {arr5.data} = {result.data}")
    
    print("\n5. 测试除法")
    arr6 = perfect_array([6, 8, 10])
    arr7 = perfect_array([2, 4, 5])
    div_result = arr6 / arr7
    print(f"  {arr6.data} / {arr7.data} = {div_result.data}")


if __name__ == "__main__":
    test_perfect_array() 